/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 17-11-2010
 * 
 */

/**
 * ConnectionManager.java
 */
package org.volverine.server;

import java.net.SocketAddress;

import org.volverine.server.event.PooledDataProcessor;
import org.volverine.server.peer.Peer;
import org.volverine.server.spi.Service;
import org.volverine.server.spi.ServiceProvider;;

/**
 * @author jzadrag
 *
 */
public class ConnectionManager implements ServiceProvider {
	private PooledDataProcessor processor;
	
	public void addPeer(Peer peer) {
		
	}
	
	public void removePeer(Peer peer) {
		
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.spi.ServiceProvider#addService(com.synergive.stealth.spi.Service)
	 */
	@Override
	public void addService(Service srv) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.spi.ServiceProvider#removeService(com.synergive.stealth.spi.Service)
	 */
	@Override
	public void removeService(Service srv) {
		// TODO Auto-generated method stub
		
	}
	
	public Connection open(SocketAddress target) {
		// TODO implement me
		return null;
	}
}

private class UdpConnection implements Connection {

	private final SocketAddress addr;
	private ConnectionState state = ConnectionState.IDLE;
	protected LinkedList<ByteChunk> queue = new LinkedList<ByteChunk>();

	public UdpConnection(SocketAddress addr) {
		this.addr = addr;
		connections.put(addr.toString(), this);
	}

	protected void datagramReceived(ByteChunk buff) {
		if (buff != null)
			synchronized (queue) {
				if(state==ConnectionState.IDLE) state=ConnectionState.OPEN;
				queue.addLast(buff);
				queue.notifyAll();
			}
	}

	public void close() {
		connections.remove(addr.toString());
		onClosing(this);
		state = ConnectionState.CLOSED;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Connection#getInetSocketAddress()
	 */
	@Override
	public SocketAddress getSocketAddress() {
		return addr;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Connection#pending()
	 */
	@Override
	public boolean pending() {
		synchronized(queue) {
			return !queue.isEmpty();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Connection#receive()
	 */
	@Override
	public ByteChunk receive() { // will block if not pending
		ByteChunk buff = null;
		while (buff == null)
			synchronized (queue) {
				try {
					buff = queue.removeFirst();
				} catch (NoSuchElementException ex) { // assume that queue
														// is empty
					try {
						queue.wait(); // wait for notification
					} catch (InterruptedException ex1) {
						// ignore interrupts
					}
				}
			}
		return buff;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Connection#send(ByteSection)
	 */
	@Override
	public void send(ByteChunk buffer) throws IOException { // send through
															// peer socket
		if(state!=ConnectionState.CLOSED) {
			socket.send(new DatagramPacket(buffer.array(), buffer.getOffset(),
					buffer.getLength(), addr));
			stats.bytesSent += buffer.getLength();
			stats.framesSent++;
			state = ConnectionState.OPEN;
		}
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.Connection#getConnectionState()
	 */
	@Override
	public ConnectionState getConnectionState() {
		return state;
	}

}

private class UdpStats implements Stats {
	protected long bytesSent, bytesReceived, framesSent, framesReceived;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Stats#getBytesReceived()
	 */
	@Override
	public long getBytesReceived() {
		return bytesReceived;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Stats#getBytesSent()
	 */
	@Override
	public long getBytesSent() {
		return bytesSent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Stats#getFramesReceived()
	 */
	@Override
	public long getFramesReceived() {
		return framesReceived;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Stats#getFramesSent()
	 */
	@Override
	public long getFramesSent() {
		return framesSent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.synergive.stealth.peer.Stats#resetCounters()
	 */
	@Override
	public void resetCounters() {
		bytesSent = 0;
		bytesReceived = 0;
		framesSent = 0;
		framesReceived = 0;
	}

}
