/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 17-11-2010
 * 
 */

/**
 * GreyList.java
 */
package org.volverine.server;

import java.net.SocketAddress;
import java.util.HashMap;

/**
 * GreyList is a BlockList which temporarily blocks addresses for 60 seconds.
 * Addresses end up in a BlockList if they repeat faulty operations.
 * 
 * @author jzadrag
 *
 */
public class GreyList implements BlockList {
	private HashMap<SocketAddress, Long> blockers = new HashMap<SocketAddress, Long>();

	/* (non-Javadoc)
	 * @see com.synergive.stealth.BlockList#addBlockedAddress(java.net.SocketAddress)
	 */
	@Override
	public synchronized void addBlockedAddress(SocketAddress address) {
		if(address!=null) blockers.put(address, new Long(System.currentTimeMillis()+60000L));
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.BlockList#isAddressBlocked(java.net.SocketAddress)
	 */
	@Override
	public synchronized boolean isAddressBlocked(SocketAddress address) {
		if(blockers.containsKey(address)) {
			if(blockers.get(address).longValue()-System.currentTimeMillis()>0) return true;
			else blockers.remove(address);
		}
		return false;
	}

}
