/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 25-11-2010
 * 
 */

/**
 * ChannelEvent.java
 */
package org.volverine.server.event;

import java.net.SocketAddress;
import java.util.EventObject;

import org.volverine.util.ByteChunk;

/**
 * @author jzadrag
 *
 */
public class DataEvent extends EventObject implements Consumable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3965719994191734297L;
	private final ByteChunk data;
	private final SocketAddress address;
	private volatile boolean consumed = false;
	
	public DataEvent(Object src, SocketAddress address, ByteChunk data) {
		super(src);
		this.address = address;
		this.data = data;
	}

	/**
	 * @return the payload
	 */
	public ByteChunk getData() {
		return data;
	}

	/**
	 * @return the address
	 */
	public SocketAddress getSocketAddress() {
		return address;
	}

	/* (non-Javadoc)
	 * @see org.volverine.server.event.Consumable#consume()
	 */
	@Override
	public void consume() {
		consumed = true;
	}

	/* (non-Javadoc)
	 * @see org.volverine.server.event.Consumable#isConsumed()
	 */
	@Override
	public boolean isConsumed() {
		return consumed;
	}

}
