/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 27-10-2010
 * 
 */

/**
 * UdpPeer.java
 */
package org.volverine.server.peer;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketAddress;
import java.net.SocketException;

import org.volverine.server.event.DataEvent;
import org.volverine.server.event.DataEventProcessor;
import org.volverine.util.ByteChunk;

/**
 * @author Jacek Zadr�g
 * 
 */
public class UdpPeer implements Peer {
	private class DatagramReceiver implements Runnable {
		private volatile boolean running = false;
		protected Thread worker;

		public DatagramReceiver() {
			worker = new Thread(this, this.getClass().getName());
			try {
				worker.setPriority(8);
			} catch (SecurityException ex) {
				System.out.println("Thread " + worker.getName()
						+ " is unable to set priority.");
				ex.printStackTrace();
			}
			worker.setDaemon(true);
			worker.start();
		}

		public synchronized boolean isRunning() {
			return running;
		}

		public void run() {
			running = true;
			while (running) {
				try {
					if (isBound()) {
						try {
							socket.receive(tmpPacket);
//							UdpConnection conn = getConnection(tmpPacket
//									.getSocketAddress());
							if(eventProcessor!=null) {
								eventProcessor.postDataEvent(new DataEvent(this, tmpPacket.getSocketAddress(), ByteChunk.wrap(
										tmpPacket.getData(), tmpPacket.getOffset(),
										tmpPacket.getLength()).copy()));
							}
//							conn.datagramReceived();
//							stats.bytesReceived += tmpPacket.getLength();
//							stats.framesReceived++;
						} catch (IOException e) {
							e.printStackTrace();
						}
					} else
						synchronized (this) {
							try {
								wait(1000); // nothing to do, sleep for a while
							} catch (InterruptedException e) {
								running = false;
							}
						}
				} catch (Exception ex) {
					ex.printStackTrace();
					// running = false;
				}
			}
		}

		public void stopRunning() {
			if (isRunning()) {
				running = false;
				unbind();
			}
		}
	}
//	private static HashMap<String, UdpConnection> connections = new HashMap<String, UdpConnection>();
	public static final int BUFFER_SIZE = 2048;
	public static final int RECEIVE_SIZE = 256;
	public static final int SEND_SIZE = 256;
	public static final int TRAFFIC_CLASS = 0x18; // LOW_DELAY + THROUGHPUT
//	private ArrayList<ConnectionListener> listeners = new ArrayList<ConnectionListener>();
	private DatagramReceiver receiver = new DatagramReceiver();

	// private byte[] temp = new byte[BUFFER_SIZE];
	private DatagramSocket socket;

//	private UdpStats stats = new UdpStats();

	private DatagramPacket tmpPacket = new DatagramPacket(
			new byte[BUFFER_SIZE], BUFFER_SIZE);
	private DataEventProcessor eventProcessor;

//	private UdpConnection getConnection(SocketAddress addr) {
//		UdpConnection conn = connections.get(addr.toString());
//		if (conn == null) {
//			conn = new UdpConnection(addr);
////			invokeListeners(new ConnectionIncomingEvent(this, conn));
//			if(eventProcessor!=null) eventProcessor.postDataEvent(new ConnectionIncomingEvent(this, conn));
//		}
//		return conn;
//	}

//	private void invokeListeners(ConnectionEvent event) {
//		if (event != null)
//			synchronized (listeners) {
//				for (ConnectionListener listener : listeners)
//					if (event instanceof ConnectionIncomingEvent) {
//						listener.onIncoming(event);
//					} else if (event instanceof ConnectionOutgoingEvent) {
//						listener.onOutgoing(event);
//					} else if (event instanceof ConnectionClosingEvent) {
//						listener.onClosing(event);
//					}
//			}
//	}

	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		receiver.stopRunning();
	}

//	protected void onClosing(Connection conn) {
//		//invokeListeners(new ConnectionClosingEvent(this, conn));
//		if(eventProcessor!=null) eventProcessor.postDataEvent(new ConnectionClosingEvent(this, conn));
//	}

//	public void addConnectionListener(ConnectionListener listener) {
//		if (listener != null)
//			synchronized (listeners) {
//				listeners.add(listener);
//			}
//	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.peer.Peer#bind(int)
	 */
	@Override
	public void bind(int port) throws SocketException {
		unbind();
		try {
			socket = new DatagramSocket(port);
		} catch (SocketException e) { // probably preferred port is not
										// available
			e.printStackTrace();
			socket = new DatagramSocket(); // bind to anything
		}
		try { // set socket options
			socket.setSendBufferSize(SEND_SIZE);
			socket.setReceiveBufferSize(RECEIVE_SIZE);
			socket.setTrafficClass(TRAFFIC_CLASS);
		} catch (SocketException e) { // life's not always perfect
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.peer.Peer#getPort()
	 */
	@Override
	public int getPort() {
		if (socket != null)
			return socket.getLocalPort();
		return -1;
	}

//	/* (non-Javadoc)
//	 * @see com.synergive.stealth.peer.Peer#getStats()
//	 */
//	@Override
//	public Stats getStats() {
//		return stats;
//	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.peer.Peer#isBound()
	 */
	@Override
	public boolean isBound() {
		return (socket != null && socket.isBound());
	}

//	public void removeConnectionListener(ConnectionListener listener) {
//		if (listener != null)
//			synchronized (listeners) {
//				listeners.remove(listener);
//			}
//	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.peer.Peer#unbind()
	 */
	@Override
	public void unbind() {
		if (socket != null)
			socket.close();
	}

//	/* (non-Javadoc)
//	 * @see com.synergive.stealth.peer.Peer#startConnection(java.net.SocketAddress)
//	 */
//	@Override
//	public Connection open(SocketAddress addr) {
//		UdpConnection conn = connections.get(addr.toString());
//		if (conn == null) {
//			conn = new UdpConnection(addr);
////			invokeListeners(new ConnectionOutgoingEvent(this, conn));
//			if(eventProcessor!=null) eventProcessor.postDataEvent(new ConnectionOutgoingEvent(this, conn));
//		}
//		return conn;
//	}

	/* (non-Javadoc)
	 * @see com.synergive.stealth.peer.Peer#setEventBus(com.synergive.stealth.event.EventBus)
	 */
	@Override
	public void setDataEventProcessor(DataEventProcessor ebus) {
		eventProcessor = ebus;
	}

	/* (non-Javadoc)
	 * @see org.volverine.server.event.DataEventHandler#getSocketAddress()
	 */
	@Override
	public SocketAddress getSocketAddress() {
		return null; // always return null
	}

	/**
	 * Sends the datagram to destination address
	 * @see org.volverine.server.event.DataEventHandler#handle(org.volverine.server.event.DataEvent)
	 */
	@Override
	public void handle(DataEvent event) {
		// TODO Think about multiple peers, who sends what?
		if(socket!=null) {
			try {
				socket.send(new DatagramPacket(event.getData().asByteArray(), 0, event.getSocketAddress()));
				event.consume();
			} catch (SocketException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		}
	}

}
