/*
 * Copyright (c) 2010 JAVART Sp. z o.o.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Created by jzadrag on 25-11-2010
 * 
 */

/**
 * EBMLTool.java
 */
package org.volverine.util;

import java.io.UnsupportedEncodingException;

/**
 * @author jzadrag
 * 
 */
public class EBMLTool {
	/**
	 * 
	 */
	private static final long serialVersionUID = -9075785022831828044L;
	private static final byte[] LENGTH_MASK = { (byte) 0x80, 0x40, 0x20, 0x10,
			0x08, 0x04, 0x02, 0x01 };
	private static final long[] VALUE_MASK = { 0x7fL, 0x3fffL, 0x1fffffL,
			0x0fffffffL, 0x07ffffffffL, 0x03ffffffffffL, 0x01ffffffffffffL,
			0x00ffffffffffffffL };
	public static final long MAX_VALUE = 0x00fffffffffffffeL;
	public static final long MIN_VALUE = 0;

	private EBMLTool() {
	} // prevent from instantiation

	/**
	 * Pushes VariableBitNumber to a ByteChunk
	 * 
	 * @param val
	 *            positive value to write
	 * @param chunk
	 *            ByteChunk to store bytes
	 * @return number of bytes written
	 */
	public static int pushVariableBitNumber(long val, ByteChunk chunk) {
		if (val > MAX_VALUE || val < MIN_VALUE)
			throw new IllegalArgumentException();
		int size = 0;
		for (; size < VALUE_MASK.length; size++) {
			if ((val & VALUE_MASK[size]) == val)
				break;
		}
		long value = LENGTH_MASK[size] & 0x00ffL; // to prevent sign expansion
		value <<= 8 * size;
		value |= val;
		byte[] temp = new byte[size + 1];
		for (int i = size; i >= 0; i--) {
			temp[i] = (byte) (value & 0x00ffL);
			value >>= 8;
		}
		chunk.writeByteArray(temp);
		return temp.length;
	}

	/**
	 * Pulls VariableBitNumber from a ByteChunk
	 * 
	 * @param chunk
	 *            ByteChunk to pull bytes from
	 * @return number
	 */
	public static long pullVariableBitNumber(ByteChunk chunk) {
		byte b = chunk.readByte();
		int size = 0;
		for (; size < LENGTH_MASK.length; size++) {
			if ((b & LENGTH_MASK[size]) != 0)
				break;
		}
		long value = b & 0x00ffL;
		for (int i = 0; i < size; i++) {
			value <<= 8;
			value |= (chunk.readByte() & 0x00ffL);
		}
		value &= VALUE_MASK[size];
		return value;
	}

	public static int pushString(String strObj, ByteChunk chunk) {
		int size = pushVariableBitNumber(strObj.length(), chunk);
		byte[] values;
		try {
			values = strObj.getBytes("UTF-8");
		} catch (UnsupportedEncodingException e) {
			values = strObj.getBytes();
		}
		chunk.writeByteArray(values);
		return size + values.length;
	}

	public static String pullString(ByteChunk chunk) {
		int size = (int) pullVariableBitNumber(chunk);
		try {
			return new String(chunk.readByteArray(size), "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return new String(chunk.readByteArray(size));
		}
	}

}
